#include "mainwindow.h"
#include "ui_mainwindow.h"

#include <QFileDialog>
#include <QMessageBox>

MainWindow::MainWindow(QWidget *parent) : QDialog(parent), ui(new Ui::MainWindow)
{
    ui->setupUi(this);
}

MainWindow::~MainWindow()
{
    delete ui;
}

void MainWindow::on_frameSlider_valueChanged(int value)
{
    ui->glWidget->setFrameOffset(value);
}

void MainWindow::on_pushButton_LoadVideo_clicked()
{
    QApplication::setOverrideCursor(Qt::WaitCursor);
    VFLResult vr = ui->glWidget->loadVideoFile(ui->lineEdit_Settings_StartFrame->text().toInt(),
                                ui->lineEdit_Settings_StepSize->text().toInt(),
                                ui->lineEdit_Settings_EndFrame->text().toInt());

    if (vr != VFL_OK)
    {
        QMessageBox::critical(this, "Error", "Couldn't load video file!");
        QApplication::restoreOverrideCursor();
    }

    int frameCount = ui->glWidget->getFrameCount();
    ui->labelSlider_1->setText(QString::number(frameCount));
    ui->frameSlider->setRange(0, frameCount - 1);
    ui->frameSlider->setEnabled(true);
    ui->labelSlider_0->setEnabled(true);
    ui->labelSlider_1->setEnabled(true);
    ui->groupBox_Visualization->setEnabled(true);
    QApplication::restoreOverrideCursor();
}

void MainWindow::on_radioButton_Style_HorseShoe_toggled(bool checked)
{
    if (checked)
    {
        ui->glWidget->setStyle(true);
        ui->groupBox_BoxSize->setEnabled(false);
    }
    else
    {
        ui->glWidget->setStyle(false);
        ui->groupBox_BoxSize->setEnabled(true);
    }

}

void MainWindow::on_radioButton_Style_Block_toggled(bool checked)
{

}


void MainWindow::on_lineEdit_StepSize_editingFinished()
{
    float stepSize = ui->lineEdit_StepSize->text().toFloat();

    ui->glWidget->setStepSize(stepSize);

    if (stepSize >= 0.01)
        ui->horizontalSlider_StepSize->setValue(1);
    else
        if (stepSize <= 0.0001)
            ui->horizontalSlider_StepSize->setValue(100);
        else
            ui->horizontalSlider_StepSize->setValue(ceil(0.01 / stepSize - 0.5));
}

void MainWindow::on_lineEdit_BoxSize_editingFinished()
{
    int boxSize = ui->lineEdit_BoxSize->text().toInt();

    ui->horizontalSlider_BoxSize->setValue(boxSize);
}

void MainWindow::on_horizontalSlider_StepSize_valueChanged(int value)
{
    ui->lineEdit_StepSize->setText(QString::number(floor(pow(10.0, 3) / (float) value + 0.5) / pow(10.0, 5)));
}

void MainWindow::on_horizontalSlider_StepSize_sliderReleased()
{
    ui->glWidget->setStepSize(ui->lineEdit_StepSize->text().toFloat());
}

void MainWindow::on_horizontalSlider_BoxSize_valueChanged(int value)
{
    ui->lineEdit_BoxSize->setText(QString::number(value));
    ui->glWidget->setBoxSize(value);
}

void MainWindow::on_radioButton_Metric_None_toggled(bool checked)
{
    if (checked)
    {
        QApplication::setOverrideCursor(Qt::WaitCursor);
        ui->glWidget->setMetric(VV_METRIC_NONE);
        QApplication::restoreOverrideCursor();
    }
}

void MainWindow::on_radioButton_Metric_YDIF_toggled(bool checked)
{
    if (checked)
    {
        QApplication::setOverrideCursor(Qt::WaitCursor);
        ui->glWidget->setMetric(VV_METRIC_YDIF);
        QApplication::restoreOverrideCursor();
    }
}

void MainWindow::on_radioButton_Metric_YMSE_toggled(bool checked)
{
    if (checked)
    {
        QApplication::setOverrideCursor(Qt::WaitCursor);
        ui->glWidget->setMetric(VV_METRIC_YMSE);
        QApplication::restoreOverrideCursor();
    }
}

void MainWindow::on_radioButton_Metric_YNDIF_toggled(bool checked)
{
    if (checked)
    {
        QApplication::setOverrideCursor(Qt::WaitCursor);
        ui->glWidget->setMetric(VV_METRIC_YNDIF);
        QApplication::restoreOverrideCursor();
    }
}

void MainWindow::on_radioButton_Metric_YNMSE_toggled(bool checked)
{
    if (checked)
    {
        QApplication::setOverrideCursor(Qt::WaitCursor);
        ui->glWidget->setMetric(VV_METRIC_YNMSE);
        QApplication::restoreOverrideCursor();
    }
}

void MainWindow::on_radioButton_Metric_IQDIF_toggled(bool checked)
{
    if (checked)
    {
        QApplication::setOverrideCursor(Qt::WaitCursor);
        ui->glWidget->setMetric(VV_METRIC_IQDIF);
        QApplication::restoreOverrideCursor();
    }
}

void MainWindow::on_pushButton_SelectFile_clicked()
{
    QString videoPath = QFileDialog::getOpenFileName(this, tr("Open Video File"), "", tr("Video files (*.avi *.wmv *.mpq *.mpeq *.mkv);;All files (*.*)"));

    if (videoPath.count() == 0)
        return;

    int hRes = 0, vRes = 0;

     QApplication::setOverrideCursor(Qt::WaitCursor);

    if (VFL_OK != ui->glWidget->openVideoFile(videoPath, &hRes, &vRes))
    {
        QMessageBox::critical(this, "Error", "Couldn't open video file!");
        QApplication::restoreOverrideCursor();
        return;
    }

    ui->label_Info_Path_Value->setText(videoPath);
    ui->label_Info_HRes_Value->setText(QString::number(hRes));
    ui->label_Info_VRes_Value->setText(QString::number(vRes));
    ui->pushButton_LoadVideo->setEnabled(true);
    QApplication::restoreOverrideCursor();
}

void MainWindow::on_transferBox_valueChanged(float * value)
{
    ui->glWidget->setTransferFunction((GLfloat *) value, 150);
}
