#include "transferbox.h"
#include <QPainter>
#include <QDebug>
#include <QMouseEvent>
#include <QVector4D>
#include <vector>

TransferBox::TransferBox(QWidget *parent) :
    QWidget(parent) {
    setAttribute(Qt::WA_NoBackground);


}

void TransferBox::paintEvent(QPaintEvent *) {
    static bool first = true;
    if(first) {
        m_image = QImage(sizeHint(), QImage::Format_RGB32);
        QPolygonF points;
        points << QPointF(0, sizeHint().height())
               << QPointF(sizeHint().width(), sizeHint().height());
        m_points = points;
        m_hoverPoints = new HoverPoints(this, HoverPoints::CircleShape);
        m_hoverPoints->setPoints(points);
        m_hoverPoints->setPointLock(0, HoverPoints::LockToLeft);
        m_hoverPoints->setPointLock(1, HoverPoints::LockToRight);
        m_hoverPoints->setSortType(HoverPoints::XSort);
        m_hoverPoints->setEditable(true);
        m_hoverPoints->setConnectionType(HoverPoints::LineConnection);

        m_colors.append(QColor(0, 0, 0));
        m_colors.append(QColor(0, 0, 0));
        first = false;
    }
    QPainter painter(this);
    painter.drawImage(0, 0, m_image);
    //draw BoundingBox
    painter.setPen(QColor(10, 10, 10));
    painter.drawRect(0, 0, sizeHint().width() - 1, sizeHint().height() - 1);
    QLinearGradient gradient(0, 0, sizeHint().width(), 0);
    for(int i = 0; i < m_points.size(); i++) {
        gradient.setColorAt((double)m_points.at(i).x()/(double)sizeHint().width(), m_colors.at(i));
    }
    painter.fillRect(rect(), gradient);
    emit valueChanged(toFloatArray());
}

void TransferBox::mousePressEvent(QMouseEvent *event) {

}

void TransferBox::addColor(QPolygonF points, QColor color) {
    bool last = true;
    for(int i = 0; i < m_points.size(); i++) {
        if(m_points.at(i).x() != points.at(i).x()) {
            m_colors.insert(i, color);
            last = false;
            break;
        }
    }
    if(last) {
        m_colors.append(color);
    }
    m_points = points;
    repaint();
}

void TransferBox::deleteColor(QPolygonF points) {
    bool last = true;
    for(int i = 0; i < points.size(); i++) {
        if(m_points.at(i).x() != points.at(i).x()) {
            m_colors.remove(i);
            last = false;
            break;
        }
    }
    if(last) {
        m_colors.remove(m_points.size()-2);
    }
    m_points = points;
    repaint();
}

void TransferBox::changeColor(int index, QColor color) {
    m_colors.remove(index);
    m_colors.insert(index, color);
    repaint();
    qDebug() << m_points.size() << " " << m_colors.size();
}

QVector<QVector4D> TransferBox::interpolate() {
    QVector<QVector4D> out;
    float stepSize = (float)width()/(STEPS-1);
    for(float step = 0; step < width(); step += stepSize) {
        int index;
        for(index = 0; index < m_points.size()-1; index++) {
            if(step < m_points.at(index+1).x()) {
                break;
            }
        }
        int nextIndex = index+1;
        float delta = m_points.at(nextIndex).x() - m_points.at(index).x();
        float factor = (step - m_points.at(index).x()) / delta;
        QVector4D vector;
        float deltaR = m_colors.at(nextIndex).red() - m_colors.at(index).red();
        float deltaG = m_colors.at(nextIndex).green() - m_colors.at(index).green();
        float deltaB = m_colors.at(nextIndex).blue() - m_colors.at(index).blue();
        float deltaA = m_points.at(nextIndex).y() - m_points.at(index).y();
        float interpolR = (m_colors.at(index).red() + (factor * deltaR))/255.0;
        float interpolG = (m_colors.at(index).green() + (factor * deltaG))/255.0;
        float interpolB = (m_colors.at(index).blue() + (factor * deltaB))/255.0;
        float interpolA = (1.0 - (m_points.at(index).y() + (factor * deltaA))/sizeHint().height());
        vector.setX(interpolR);
        vector.setY(interpolG);
        vector.setZ(interpolB);
        vector.setW(interpolA);
        out.append(vector);
    }
    //append last
    float step = width();
    int index = m_points.size() - 2;
    int nextIndex = m_points.size() - 1;
    float delta = m_points.at(nextIndex).x() - m_points.at(index).x();
    float factor = 1.0;
    QVector4D vector;
    float deltaR = m_colors.at(nextIndex).red() - m_colors.at(index).red();
    float deltaG = m_colors.at(nextIndex).green() - m_colors.at(index).green();
    float deltaB = m_colors.at(nextIndex).blue() - m_colors.at(index).blue();
    float deltaA = m_points.at(nextIndex).y() - m_points.at(index).y();
    float interpolR = (m_colors.at(index).red() + (factor * deltaR))/255.0;
    float interpolG = (m_colors.at(index).green() + (factor * deltaG))/255.0;
    float interpolB = (m_colors.at(index).blue() + (factor * deltaB))/255.0;
    float interpolA = (1.0 - (m_points.at(index).y() + (factor * deltaA))/sizeHint().height());
    vector.setX(interpolR);
    vector.setY(interpolG);
    vector.setZ(interpolB);
    vector.setW(interpolA);
    out.append(vector);
    return out;
}

float* TransferBox::toFloatArray() {
    float* value = new float[STEPS * 4];
    QVector<QVector4D> vector = interpolate();
    for (int i=0; i < vector.count(); i++) {
        QVector4D point = vector.at(i);
        value[i * 4] = point.x();
        value[i * 4 + 1] = point.y();
        value[i * 4 + 2] = point.z();
        value[i * 4 + 3] = pow(point.w(), 8);
    }
    return value;
}
