#ifndef TRANSFERBOX_H
#define TRANSFERBOX_H

#include <QWidget>
#include <QColorDialog>
#include "hoverpoints.h"

#define STEPS 150

//! The TransferBox widget class
/*!
 * This class renders the transfer function. Every Point from m_points has a corresponding color that gets rendered when a paintEvent occurs.
 * Points are managed, displayed and returned by the HoverPoints class. The widget is drawn with a QLinearGradient. Users can select the colors from a
 * QColorDialog.
 */

class TransferBox : public QWidget
{
    Q_OBJECT
public:
    enum ShadeType {
        RedShade,
        GreenShade,
        BlueShade
    };

    explicit TransferBox(QWidget *parent = 0);
    QSize sizeHint() const {return QSize(300, 100); }
    void setPoints(QPolygonF points) {m_points = points;}
    /*!
     * add a color to m_colors
     */
    void addColor(QPolygonF points, QColor color);
    /*!
     * delete a color from m_colors
     */
    void deleteColor(QPolygonF points);
    /*!
     * change a color from m_colors by removing and inserting one.
     */
    void changeColor(int index, QColor color);
signals:
    void valueChanged(float *value);
public slots:

private:
    void paintEvent(QPaintEvent *event);
    void mousePressEvent(QMouseEvent *event);
    /*!
     * interpolates linearly between the colors and the opacity. gets called by the toFloatArray() function.
     * returns a QVector of QVector4D colors.
     */
    QVector<QVector4D> interpolate();
    /*!
     * converts the QVector to a float array, which can be sent to the GPU. gets called when a valueChanged signal is emitted.
     */
    float* toFloatArray();

    QImage m_image;
    QColorDialog m_dialog;
    HoverPoints *m_hoverPoints;
    QVector<QColor> m_colors;
    QPolygonF m_points;
    
};

#endif // TRANSFERBOX_H
