#version 330

in vec2 vTexCoord;
in vec4 vPos;

layout(location = 0, index = 0) out vec4 fragColor;

uniform sampler3D u_videoTexture;
uniform sampler1D u_meanTexture;
uniform sampler1D u_devTexture;
uniform int u_frameCount;
uniform int u_currentFrame;
uniform bool u_nTrans;

const float PI = 3.14159265358979323846264;
const mat3 colorTrans = mat3(0.299, 0.596, 0.212, 0.587, -0.275, -0.523, 0.144, -0.321, 0.311);


void main(void)
{
    vec3 firstFrameCoord;
    vec3 secondFrameCoord;

    firstFrameCoord.xy = vTexCoord;
    secondFrameCoord.xy = vTexCoord;

    firstFrameCoord.z = (float(u_currentFrame) + 0.5) / float(u_frameCount);

    if (u_currentFrame != u_frameCount - 1)
    {
        secondFrameCoord.z = (float(u_currentFrame) + 1.5) / float(u_frameCount);
    }
    else
    {
        secondFrameCoord.z = 0.5 / float(u_frameCount);
    }

    vec3 firstFrameYIQ = colorTrans * vec3(texture(u_videoTexture, firstFrameCoord));
    vec3 secondFrameYIQ = colorTrans * vec3(texture(u_videoTexture, secondFrameCoord));

    if (u_nTrans)
    {
        firstFrameYIQ.r = (firstFrameYIQ.r - texture(u_meanTexture, firstFrameCoord.z).r) / texture(u_devTexture, firstFrameCoord.z).r;
        secondFrameYIQ.r = (secondFrameYIQ.r - texture(u_meanTexture, secondFrameCoord.z).r) / texture(u_devTexture, secondFrameCoord.z).r;
    }

    float YDif = abs(firstFrameYIQ.r - secondFrameYIQ.r);

    vec4 outColor;

    if (!u_nTrans)
    {
        float IQDif = dot(normalize(firstFrameYIQ.gb), normalize(secondFrameYIQ.gb));

        if (IQDif < 0)
            IQDif = 0.5 - IQDif/2.0;
        else
            IQDif = (1 - IQDif)/2.0;

        outColor = vec4(YDif, IQDif, 0.0, 1.0);
    }
    else
    {
        outColor = vec4(YDif, 0.0, 0.0, 1.0);
    }

    fragColor = outColor;
    //fragColor = texture(u_videoTexture, firstFrameCoord);
}
