#version 330

in vec3 vTexCoord;
in vec4 vPos;

uniform sampler3D u_videoTexture;
uniform sampler1D u_transferTexture;
uniform float u_frameOffset;
uniform mat4 u_invM;
uniform vec3 u_cameraPos;
uniform vec3 u_backColor;
uniform bool u_horseShoeView;
uniform float u_stepSize;
uniform int u_metric;

const float PI = 3.14159265358979323846264;
const mat3 colorTrans = mat3(0.299, 0.596, 0.212, 0.587, -0.275, -0.523, 0.144, -0.321, 0.311);


vec3 horseShoeTrans(vec3 texCoord);
vec3 result;
layout(location = 0, index = 0) out vec4 fragColor;

void main(void)
{
    vec3 worldPos = vec3(vPos);

    vec3 rayStartTC = vTexCoord;
    vec3 rayDir = normalize(worldPos - u_cameraPos);

    rayDir = normalize((u_invM * vec4(rayDir, 1.0)).xyz);

    //Turn y and z component of the rayDir around, to get into texture space directions
    rayDir.yz *= -1;

    vec3 currentSample = rayStartTC;

    vec4 outColor = vec4(0, 0, 0, 0);

    int maxSteps = int(ceil(sqrt(3.0) / u_stepSize));

    for (int stepCounter=0; stepCounter < maxSteps; stepCounter++)
    {
        if (currentSample.x < 0 || currentSample.x > 1 ||
                currentSample.y < 0 || currentSample.y > 1 ||
                currentSample.z < 0 || currentSample.z > 1)
            break;


        vec3 correctedSample = currentSample;

        if (u_horseShoeView)
        {
            correctedSample = horseShoeTrans(correctedSample);

            if (correctedSample.x < 0)
            {
                currentSample += rayDir * u_stepSize;
                continue;
            }
        }

        correctedSample.z += u_frameOffset;

        if (correctedSample.z > 1.0)
            correctedSample.z -= 1;

        vec3 sampleColor = texture(u_videoTexture, correctedSample).rgb;

        float sampleLuminace;

        if (u_metric == 0)
            sampleLuminace = float((colorTrans * sampleColor));
        else
            if (u_metric == 1)
                sampleLuminace = sampleColor.g;
            else
                if (u_metric == 4 || u_metric == 5)
                    sampleLuminace = pow(sampleColor.r, 2);
                else
                    sampleLuminace = sampleColor.r;


        vec4 transferColor = texture(u_transferTexture, sampleLuminace);

        if (u_metric != 0)
            sampleColor.rgb = transferColor.rgb;

        float sampleAlpha = transferColor.a;

        outColor.rgb += (1 - outColor.a) * sampleColor * sampleAlpha;
        outColor.a += (1 - outColor.a) * sampleAlpha;

        if (outColor.a >= 1)
        {
            outColor.a = 1;
            break;
        }

        currentSample += rayDir * u_stepSize;
    }

    if (outColor.a < 1)
    {
        outColor.rgb += (1 - outColor.a) * u_backColor;
        outColor.a = 1;
    }

    fragColor = outColor;
}

vec3 horseShoeTrans(vec3 texCoord)
{
    float r = sqrt(pow(texCoord.x - 0.5, 2) +/* pow(texCoord.y - 0.5, 2) +*/ pow(texCoord.z/2, 2));
    float phi = atan(texCoord.z/2, texCoord.x - 0.5);

    //while (phi < -PI)
        //phi + 2 * PI;

    //while (phi > PI)
        //phi - 2 * PI;

    result = vec3(0.0, 0.0, 0.0);

    if (r >= 0.25 && r <= 0.5)
        result.x = 2.0 - (4.0 * r);
    else
        return vec3(-1.0, 0.0, 0.0);

    result.y = texCoord.y;

    if (phi >= 0)
        result.z = 1.0 - phi/PI;
    else
        return vec3(-1.0,0.0,0.0);


    return result;
}
